unit RingGroupDlg;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs,

  VaxTeleServerSIP, Vcl.ExtCtrls, Vcl.StdCtrls;

type
    TRingGroupForm = class(TForm)

    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    ComboRingGroupName: TComboBox;
    Label2: TLabel;
    ComboProcessMode: TComboBox;
    GroupBox3: TGroupBox;
    RBTN_free: TRadioButton;
    RBTN_Any: TRadioButton;
    Label3: TLabel;
    ListRingGroupAgent: TListBox;
    BtnAddAgent: TButton;
    BtnRemoveAgent: TButton;
    BtnClearAgent: TButton;
    BtnAddRingGroup: TButton;
    BtnRemoveRingGroup: TButton;
    BtnClearRingGroup: TButton;
    ComboLogin: TComboBox;
    procedure BtnRemoveRingGroupClick(Sender: TObject);
    procedure BtnClearRingGroupClick(Sender: TObject);
    procedure BtnAddAgentClick(Sender: TObject);
    procedure BtnRemoveAgentClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure BtnAddRingGroupClick(Sender: TObject);
    procedure BtnClearAgentClick(Sender: TObject);
    procedure ComboRingGroupNameChange(Sender: TObject);

 private


    function AddRingGroupChecks(): boolean;
    function GetProcessMood(): Integer;
    function GetModeType() : Integer;

    procedure FillLogin();
    procedure FillRingGroup();

    procedure SetProcessMode();
    procedure ClearDlgContent();

    procedure UpdateDlgContent();

  public

    procedure OnSetActive();
    constructor Create(AOwner : TComponent; var pVaxTeleSIP : TVaxTeleServerSIP); reintroduce; overload;

  end;

var
  RingGroupForm: TRingGroupForm;
  m_pVaxTeleSIP : TVaxTeleServerSIP;

implementation

{$R *.dfm} 
procedure TRingGroupForm.OnSetActive();
begin
    UpdateDlgContent();
end;

procedure TRingGroupForm.BtnAddAgentClick(Sender: TObject);
var
sAgentName : string;
begin
      sAgentName := ComboLogin.Text;
      ListRingGroupAgent.Items.Add(sAgentName);
    
end;

procedure TRingGroupForm.BtnAddRingGroupClick(Sender: TObject);
var
    sRingGroupAgent : string;
    nIndex : Integer;
begin

	if(AddRingGroupChecks() = False) Then
  begin
    Exit();
  end;


	m_pVaxTeleSIP.RemoveRingGroup(ComboRingGroupName.Text);

   if(m_pVaxTeleSIP.AddRingGroup(ComboRingGroupName.Text) <> True) then
   begin
     ShowMessage('Ring Group could not be added.');
     Exit();
   end;


	for sRingGroupAgent in ListRingGroupAgent.Items do
  begin
         m_pVaxTeleSIP.AddRingGroupAgent(ComboRingGroupName.Text, sRingGroupAgent);
  end;

  ComboRingGroupName.Items.Add(ComboRingGroupName.Text);
  m_pVaxTeleSIP.SetRingGroupProcessMode(ComboRingGroupName.Text, GetProcessMood(), GetModeType());

  ShowMessage('RingGroup created successfully"');

end;


function TRingGroupForm.AddRingGroupChecks(): boolean;
begin

  if(ComboRingGroupName.Text = '') then
  begin
    ShowMessage('Please enter RingGroup Name.');
    Exit();
  end;

  if(ComboProcessMode.Text = '') then
  begin
    ShowMessage('Please select Process Mode');
    Exit();
  end;

  if((RBTN_free.Checked = False) and (RBTN_Any.Checked = False))  then
  begin
    ShowMessage('Please select Process Mode Type Free/Any');
    Exit(False);
  end;

  Result := True;

end;

function TRingGroupForm.GetProcessMood(): Integer;
begin
   Result := ComboProcessMode.ItemIndex;
end;

function TRingGroupForm.GetModeType() : Integer;
begin

  if(RBTN_free.Checked)  then
    begin
      Exit(1);
    end;

   Result := 2;
end;


procedure TRingGroupForm.FillLogin();
var
  objLoginList : TStringList;
  sAgentName : string;

begin
  ComboLogin.Clear();
  objLoginList := TStringList.Create();
  m_pVaxTeleSIP.GetUserAll(objLoginList);

  for sAgentName in objLoginList do
    begin
      ComboLogin.Items.Add(sAgentName);
    end;

    ComboLogin.ItemIndex := 0;
end;

procedure TRingGroupForm.FillRingGroup();
var
  objRingGroupList : TStringList;
  sRingGroupName : string;

begin

  objRingGroupList := TStringList.Create();
  m_pVaxTeleSIP.GetRingGroupList(objRingGroupList);

  ComboRingGroupName.Clear();
  for sRingGroupName in objRingGroupList do
    begin
      ComboRingGroupName.Items.Add(sRingGroupName);
    end;

    ComboRingGroupName.ItemIndex := 0;
end;

procedure TRingGroupForm.SetProcessMode();
begin

   ComboProcessMode.Clear();
   ComboProcessMode.Items.Add('Ring All');
   ComboProcessMode.Items.Add('Hunt Random');
   ComboProcessMode.Items.Add('Round Robin');
   ComboProcessMode.Items.Add('Least Call Count');
   ComboProcessMode.Items.Add('Least Talk Time');
   ComboProcessMode.Items.Add('Longest Waiting');
   ComboProcessMode.Items.Add('Prioritized Hunt');

   ComboProcessMode.ItemIndex := 0;
end;

procedure TRingGroupForm.BtnClearAgentClick(Sender: TObject);
begin
      ListRingGroupAgent.Clear();
end;

procedure TRingGroupForm.BtnClearRingGroupClick(Sender: TObject);
begin
   ClearDlgContent();
end;

procedure TRingGroupForm.BtnRemoveAgentClick(Sender: TObject);
var
nIndex: Integer;
sAgentName  : string;
begin

    for nIndex:= ListRingGroupAgent.items.count -1 downto 0 do
      if ListRingGroupAgent.selected[nIndex] then
        ListRingGroupAgent.items.delete(nIndex);
     
end;

procedure TRingGroupForm.BtnRemoveRingGroupClick(Sender: TObject);

begin

   m_pVaxTeleSIP.RemoveRingGroup(ComboRingGroupName.Text);
   ClearDlgContent();

   ComboRingGroupName.items.delete(ComboRingGroupName.ItemIndex);
   ShowMessage('Ring Group Removed Successfully!');
end;

constructor TRingGroupForm.create(AOwner : TComponent; var pVaxTeleSIP : TVaxTeleServerSIP);
begin
        inherited Create(AOwner);
        m_pVaxTeleSIP := pVaxTeleSIP;
    end;

procedure TRingGroupForm.FormActivate(Sender: TObject);
begin
       ShowMessage('RingGroup');
end;

procedure TRingGroupForm.UpdateDlgContent();
begin

   FillRingGroup();
   FillLogin();
   SetProcessMode();

end;

procedure TRingGroupForm.ClearDlgContent();
begin

   ComboRingGroupName.Text := '';
   ComboProcessMode.Text := '';

   ListRingGroupAgent.Clear();

end;

procedure TRingGroupForm.ComboRingGroupNameChange(Sender: TObject);
var
  slistAgentName : TStringList;
  sAgentName : string;
  nModeId : Integer;
  nModeType : Integer;

begin
        if(ComboRingGroupName.Text = '') then
        begin
          Exit();
        end;

        slistAgentName := TStringList.Create();

       if(m_pVaxTeleSIP.GetRingGroupDetails(ComboRingGroupName.Text, nModeId, nModeType, slistAgentName) = False) then
       begin
         Exit();
       end;

       ListRingGroupAgent.Clear();
       for sAgentName in slistAgentName do
        begin
         ListRingGroupAgent.Items.Add(sAgentName);
        end;

        ComboProcessMode.ItemIndex := nModeId;

        RBTN_free.Checked := False;
        RBTN_Any.Checked := False;

        if(nModeType = 1) then
          RBTN_free.Checked := True;
        if(nModeType = 2) then
           RBTN_Any.Checked := True;

end;

end.
