// QueueDlg.cpp : implementation file
//

#include "stdafx.h"
#include "VaxTeleServerApp.h"
#include "QueueDlg.h"
#include "afxDialogEx.h"

#include"VaxServerSIP.h"
#include"VaxServerExCOM.h"

#include"Define.h"

// CQueueDlg dialog

IMPLEMENT_DYNAMIC(CQueueDlg, CPropertyPage)

CQueueDlg::CQueueDlg(CVaxServerSIP* pVaxTeleSIP): CPropertyPage(CQueueDlg::IDD)
{
	m_pVaxTeleSIP = pVaxTeleSIP;
	
	m_sQueueName = "";
	m_sQueueAgentName = "";	

	m_sProcessMode = "";
	m_sProcessType = _T("");

	m_sQueueMusicFile = _T("..\\WaveFiles\\Queue.wav");
	m_sQueueDialToneFile = _T("..\\WaveFiles\\DialTone.wav");
}

CQueueDlg::~CQueueDlg()
{
}

void CQueueDlg::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_COMBO_QUEUE, m_ctrlComboQueue);
	DDX_CBString(pDX, IDC_COMBO_QUEUE, m_sQueueName);
	DDX_Control(pDX, IDC_COMBO_MODE, m_ctrlComboProcessMode);
	DDX_Control(pDX, IDC_COMBO_QUEUE_AGENT, m_ctrlComboAgents);
	DDX_CBString(pDX, IDC_COMBO_QUEUE_AGENT, m_sQueueAgentName);
	DDX_CBString(pDX, IDC_COMBO_MODE, m_sProcessMode);
		
	DDX_Control(pDX, IDC_LIST_LOGIN, m_ctrlListQueueAgent);
	DDX_Text(pDX, IDC_EDIT_QUEUE_MUSIC, m_sQueueMusicFile);
	DDX_Text(pDX, IDC_EDIT_QUEUE_DIAL_TONE, m_sQueueDialToneFile);
	DDX_Control(pDX, IDC_COMBO_PROCESS_TYPE, m_ctrlComboProcessType);
	DDX_CBString(pDX, IDC_COMBO_PROCESS_TYPE, m_sProcessType);
}

BEGIN_MESSAGE_MAP(CQueueDlg, CPropertyPage)
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_BUTTON_ADD_QUEUE, &CQueueDlg::OnButtonQueueAdd)
	ON_BN_CLICKED(IDC_BUTTON_QUEUE_CLEAR, &CQueueDlg::OnButtonClearQueueDlg)
	ON_BN_CLICKED(IDC_BUTTON_REMOVE_QUEUE, &CQueueDlg::OnButtonQueueRemove)
	ON_BN_CLICKED(IDC_BUTTON_ADD_AGENT, &CQueueDlg::OnButtonQueueAgentAdd)
	ON_BN_CLICKED(IDC_BUTTON_REMOVE_AGENT, &CQueueDlg::OnButtonQueueAgentRemove)
	ON_BN_CLICKED(IDC_BUTTON_QUEUE_AGENT, &CQueueDlg::OnButtonClearQueueAgentDlg)
	ON_WM_CREATE()
	ON_CBN_SELCHANGE(IDC_COMBO_QUEUE, &CQueueDlg::OnCbnSelchangeComboQueue)
	
END_MESSAGE_MAP()


BOOL CQueueDlg::OnSetActive()
{
	AddProcessMode();
	AddProcessType();

	AddUsers();
	AddQueues();

	ShowQueueDetails();
	
	return CPropertyPage::OnSetActive();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CQueueDlg::OnButtonQueueAdd()
{
	UpdateData();
	
	if(m_ctrlComboQueue.FindStringExact(0, m_sQueueName) != -1)
	{
		if(MessageBox("Queue already exists, do you want to continue?", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_YESNO) == IDNO)
			return;
	}

	if(!AddQueueChecks())
		return;

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	m_pVaxTeleSIP->RemoveQueue(m_sQueueName);

	if(!m_pVaxTeleSIP->AddQueue(m_sQueueName, m_sQueueMusicFile, m_sQueueDialToneFile))
	{
		MessageBox("Queue could not be added/updated or Please start the server first.", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_OK);
		return;
	}

	CString sQueueAgent;

	for(int nIndex = 0; nIndex < m_ctrlListQueueAgent.GetCount(); nIndex++)
	{
		m_ctrlListQueueAgent.GetText(nIndex, sQueueAgent);
		m_pVaxTeleSIP->AddQueueAgent(m_sQueueName, sQueueAgent);
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	int nProcessMode = GetProcessMode();
	if(nProcessMode == -1) return;
	
	int nProcessType = GetProcessType();
	if(nProcessType == -1) return;
	
	m_pVaxTeleSIP->SetQueueProcessMode(m_sQueueName, nProcessMode, nProcessType);

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	if(m_ctrlComboQueue.FindStringExact(0, m_sQueueName) == -1)
		m_ctrlComboQueue.AddString(m_sQueueName);
		
	MessageBox("Queue added/updated successfully.", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_OK);
}

void CQueueDlg::OnButtonQueueRemove()
{
	UpdateData();
	
	int nIndex = m_ctrlComboQueue.FindStringExact(0, m_sQueueName);
	if(nIndex == -1) return;

	m_pVaxTeleSIP->RemoveQueue(m_sQueueName);
	m_ctrlComboQueue.DeleteString(nIndex);

	ClearDlgContent();
	
	if(m_ctrlComboQueue.GetCount() > 0)
		m_ctrlComboQueue.SetCurSel(0);
		
	ShowQueueDetails();
	
	MessageBox("Queue removed successfully!", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_OK);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CQueueDlg::OnButtonQueueAgentAdd()
{
	UpdateData();

	if(!m_ctrlListQueueAgent.FindString(0, m_sQueueAgentName))
	{
		MessageBox("Agent already exists.", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_OK);
		return;
	}

	m_ctrlListQueueAgent.AddString(m_sQueueAgentName);
}

void CQueueDlg::OnButtonQueueAgentRemove()
{
	int nCurIndex = m_ctrlListQueueAgent.GetCurSel();
	if(nCurIndex == -1) return;
	
	m_ctrlListQueueAgent.DeleteString(nCurIndex);
}

void CQueueDlg::OnButtonClearQueueAgentDlg()
{
	ClearAgentDlgContent();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CQueueDlg::OnCbnSelchangeComboQueue()
{
	ShowQueueDetails();
}

void CQueueDlg::OnButtonClearQueueDlg()
{
	ClearDlgContent();
}

void CQueueDlg::ClearDlgContent()
{
	m_sQueueName = "";
	m_sProcessMode = "";

	m_sQueueMusicFile = "";
	m_sQueueDialToneFile = "";

	m_sQueueAgentName = "";
	m_ctrlListQueueAgent.ResetContent();

	UpdateData(FALSE);
}

void CQueueDlg::ShowQueueDetails()
{
	UpdateData();

	int nSelectedIndex = m_ctrlComboQueue.GetCurSel();
	if(nSelectedIndex == -1) return;

	CString sQueueName;
	
	m_ctrlComboQueue.GetLBText(nSelectedIndex, sQueueName);
	if(sQueueName.IsEmpty()) return;

	m_sQueueName = sQueueName;
	
	CStringList objListAgent;
	int nProcessMode, nProcessType;

	CString sQueueMusicFile, sQueueDialToneFile;
	
	if(!m_pVaxTeleSIP->GetQueueDetails(sQueueName, nProcessMode, nProcessType, &objListAgent, sQueueMusicFile, sQueueDialToneFile))
		return;

	m_sQueueMusicFile = sQueueMusicFile;
	m_sQueueDialToneFile = sQueueDialToneFile;

	m_ctrlComboProcessMode.GetLBText(nProcessMode, m_sProcessMode);
	m_ctrlComboProcessType.GetLBText(nProcessType, m_sProcessType);

	m_ctrlListQueueAgent.ResetContent();

	while(!objListAgent.IsEmpty())
		m_ctrlListQueueAgent.AddString(objListAgent.RemoveHead());

	UpdateData(FALSE);
}

void CQueueDlg::ClearAgentDlgContent()
{
	m_ctrlListQueueAgent.ResetContent();
	m_sQueueAgentName = "";
	
	UpdateData(FALSE);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CQueueDlg::AddQueues()
{
	UpdateData();

	m_ctrlComboQueue.ResetContent();
	
	CStringList slistQueueName;
	m_pVaxTeleSIP->GetQueueNameAll(&slistQueueName);

	while(!slistQueueName.IsEmpty())
		m_ctrlComboQueue.AddString(slistQueueName.RemoveHead());

	if(m_ctrlComboQueue.GetCount() > 0)
		m_ctrlComboQueue.SetCurSel(0);
}

void CQueueDlg::AddProcessMode()
{
	m_ctrlComboProcessMode.ResetContent();

	m_ctrlComboProcessMode.AddString(USER_ALGO_HUNT_RANDOM);
	m_ctrlComboProcessMode.AddString(USER_ALGO_LEAST_ANSWERED);
	m_ctrlComboProcessMode.AddString(USER_ALGO_LEAST_TALK_TIME);
	m_ctrlComboProcessMode.AddString(USER_ALGO_LONGEST_WAITING);
	m_ctrlComboProcessMode.AddString(USER_ALGO_PRIORITIZED_HUNT);
	m_ctrlComboProcessMode.AddString(USER_ALGO_ROUND_ROBIN);

	m_ctrlComboProcessMode.SetCurSel(0);

	UpdateData(FALSE);
}

void CQueueDlg::AddProcessType()
{
	m_ctrlComboProcessType.ResetContent();

	m_ctrlComboProcessType.AddString(USER_ALGO_TYPE_FREE);
	m_ctrlComboProcessType.AddString(USER_ALGO_TYPE_ANY);

	m_ctrlComboProcessType.SetCurSel(0);

	UpdateData(FALSE);
}

void CQueueDlg::AddUsers()
{
	m_ctrlComboAgents.ResetContent();

	CStringList objListUsers;
	m_pVaxTeleSIP->GetUserAll(&objListUsers);
	
	if(objListUsers.IsEmpty())
		return;

	while(!objListUsers.IsEmpty())
	{
		CString objLogin = objListUsers.RemoveHead();
		m_ctrlComboAgents.AddString(objLogin);
	}

	m_ctrlComboAgents.SetCurSel(0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CQueueDlg::GetProcessMode()
{
	if(m_ctrlComboProcessMode.GetCurSel() == -1)
	{
		MessageBox("Select Process Mode");
		return -1;
	}

	return m_ctrlComboProcessMode.GetCurSel();
}

int CQueueDlg::GetProcessType()
{
 	if(m_ctrlComboProcessType.GetCurSel() == -1)
	{
		MessageBox("Select Process type");
		return -1;
	}

	return m_ctrlComboProcessType.GetCurSel();
}

BOOL CQueueDlg::AddQueueChecks()
{
	if(m_sQueueName.IsEmpty())
	{
		MessageBox("Please enter a Queue Name.", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_OK);
		return FALSE;
	}

	if(m_sProcessMode.IsEmpty())
	{
		MessageBox("Please select a Process Mode.", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_OK);
		return FALSE;
	}

	return TRUE;
}

void CQueueDlg::OnClose()
{
	CPropertyPage::OnClose();
}

