#include "StdAfx.h"
#include "VaxCallSession.h"
#include "VaxServerExCOM.h"
#include "VaxCall.h"
#include "VaxTransfer.h"
#include "VaxQueue.h"
#include "VaxRingGroup.h"
#include "VaxHold.h"
#include "VaxDTMF.h"
#include "VaxRecordPCM.h"
#include "VaxDialTone.h"
#include "VaxDetectedAMD.h"

#include "StealthListen.h"
#include "CallParking.h"
#include "CallPickUp.h"
#include "VaxRoom.h"

CVaxCallSession::CVaxCallSession(CVaxServerExCOM* pServerExCOM)
{
	m_pServerExCOM = pServerExCOM;
	
	m_bIsCallSessionAddedToRoom = FALSE;
	
	m_pVaxCall = new CVaxCall();
	m_pVaxTransfer = new CVaxTransfer();
	m_pVaxQueue = new CVaxQueue();
	m_pVaxRingGroup = new CVaxRingGroup();
	m_pVaxHold = new CVaxHold();
	m_pVaxDTMF = new CVaxDTMF();
	m_pVaxRecordPCM = new CVaxRecordPCM();
	m_pVaxDialTone= new CVaxDialTone();
	m_pVaxDetectedAMD = new CVaxDetectedAMD();
	m_pStealthListen = new CStealthListen();
	m_pCallParking = new CCallParking();
	m_pCallPickUp = new CCallPickUp();
	m_pVaxRoom = new CVaxRoom();

}

CVaxCallSession::~CVaxCallSession()
{
	delete m_pVaxCall;
	delete m_pVaxTransfer;
	delete m_pVaxQueue;
	delete m_pVaxRingGroup;
	delete m_pVaxHold;
	delete m_pVaxDTMF;
	delete m_pVaxRecordPCM;
	delete m_pVaxDialTone;
	delete m_pVaxDetectedAMD;

	delete m_pStealthListen;
	delete m_pCallParking;
	delete m_pCallPickUp;
	delete m_pVaxRoom;
}

////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////

void CVaxCallSession::OnIncomingCall(LONG lSessionId, LPCTSTR sCallerName, LPCTSTR sCallerId, LPCTSTR sDialNo, LPCTSTR sRoomName, LPCTSTR sQueueName, LPCTSTR sRingGroupName,  LONG lFromPeerType, LPCTSTR sFromPeerName, LONG lToPeerType, LPCTSTR sToPeerName, LPCTSTR sUserAgentName, LPCTSTR sFromIP, LONG lFromPort)
{
	BOOL bProcessed = FALSE;

	m_pVaxRoom->OnIncomingCall(lSessionId, sRoomName, bProcessed);
	
	if(bProcessed)
	{
		m_bIsCallSessionAddedToRoom = TRUE;
		return;
	}
	
	m_pVaxQueue->OnIncomingCall(lSessionId, sCallerName, sCallerId, sQueueName, bProcessed); //Try to process as Queue call.

	if(bProcessed)
		return;

	m_pVaxRingGroup->OnIncomingCall(lSessionId, sCallerName, sCallerId, sRingGroupName,  bProcessed); //Try to process as RingGroup call.

	if(bProcessed)
		return;
	
	m_pCallPickUp->OnIncomingCall(lSessionId, sDialNo, lFromPeerType, sFromPeerName, bProcessed); //A user wants to pickup other user/extention's call.

	if(bProcessed)
		return;

	m_pCallParking->OnIncomingCall(lSessionId, sDialNo, lFromPeerType,  bProcessed);  //Try to connect to Parked call.

	if(bProcessed)
		return;

	m_pStealthListen->OnIncomingCall(lSessionId, sDialNo, lFromPeerType,  bProcessed); //Process for stealth-listening.

	if(bProcessed)
		return;

	m_pVaxCall->OnIncomingCall(lSessionId, sCallerName, sCallerId, sDialNo, lFromPeerType, sFromPeerName,  lToPeerType, sToPeerName, bProcessed); // 'Process as normal call.
}

////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////

void CVaxCallSession::OnCallSessionFailed(LONG lSessionId, int nChannelId, LONG lStatusCode, LPCTSTR sReasonPhrase)
{
	m_pVaxCall->OnCallSessionFailed(lSessionId, nChannelId, lStatusCode, sReasonPhrase);
}

void CVaxCallSession::OnCallSessionConnecting(LONG lSessionId, int nChannelId, LONG lStatusCode, LPCTSTR sReasonPhrase)
{
	m_pVaxCall->OnCallSessionConnecting(lSessionId, nChannelId, lStatusCode, sReasonPhrase);
}

void CVaxCallSession::OnCallSessionConnected(LONG lSessionId)
{
	if(m_bIsCallSessionAddedToRoom)
	{
		m_pVaxRoom->OnCallSessionConnected(lSessionId);
		return;
	}

	m_pVaxCall->OnCallSessionConnected(lSessionId);
}

void CVaxCallSession::OnCallSessionLost(LONG lSessionId, int nChannelId)
{
	m_pVaxCall->OnCallSessionLost(lSessionId, nChannelId);
}

void CVaxCallSession::OnCallSessionHangup(LONG lSessionId, int nChannelId)
{
	m_pVaxCall->OnCallSessionHangup(lSessionId, nChannelId);
} 

void CVaxCallSession::OnCallSessionTimeout(LONG lSessionId, int nChannelId)
{
	m_pVaxCall->OnCallSessionTimeout(lSessionId, nChannelId);
}

void CVaxCallSession::OnCallSessionCancelled(LONG lSessionId)
{
	m_pVaxCall->OnCallSessionCancelled(lSessionId);
}
//////////////////////////////////////////////////////////////////////////
/////////////////////////  CVAXHOLD //////////////////////////////////

void CVaxCallSession::OnCallSessionOnHold(LONG lSessionId, int nChannelId)
{
	if(m_bIsCallSessionAddedToRoom)
	{
		m_pVaxRoom->OnCallSessionOnHold(lSessionId, nChannelId);
		return;
	}

	m_pVaxHold->OnCallSessionOnHold(lSessionId, nChannelId);
}

void CVaxCallSession::OnCallSessionOffHold(LONG lSessionId, int nChannelId)
{
	if(m_bIsCallSessionAddedToRoom)
	{
		m_pVaxRoom->OnCallSessionOffHold(lSessionId, nChannelId);
		return;
	}

	m_pVaxHold->OnCallSessionOffHold(lSessionId, nChannelId);
}

//////////////////////////////////////////////////////////////////////////
//////////////////////////  CVAXTRANSFER//////////////////////////////////

void CVaxCallSession::OnCallSessionTransferBlind(LONG lTransfererSessionId, LONG lTransfererChannelId, LPCTSTR sTransferer, LPCTSTR sTransferee, LPCTSTR sTransferTo)
{
	if(m_bIsCallSessionAddedToRoom)
	{
		m_pVaxRoom->OnCallSessionTransferBlind(lTransfererSessionId, lTransfererChannelId, sTransferer, sTransferee, sTransferTo);
		return;
	}

	BOOL bProcessed = FALSE;
		
	m_pCallParking->OnCallSessionTransferBlind(lTransfererSessionId, lTransfererChannelId, sTransferer, sTransferee, sTransferTo, bProcessed);

	if(bProcessed)
		return;

	m_pVaxTransfer->OnCallSessionTransferBlind(lTransfererSessionId, lTransfererChannelId, sTransferer, sTransferee, sTransferTo);
}

void CVaxCallSession::OnCallSessionTransferConsult(LONG lTransfererSessionId, int nTransfererChannelId, LONG lTransferToSessionId, int nTransferToChannelId, LPCTSTR sTransferer, LPCTSTR sTransferee, LPCTSTR sTransferTo)
{
	if(m_bIsCallSessionAddedToRoom)
	{
		m_pVaxRoom->OnCallSessionTransferConsult(lTransfererSessionId, nTransfererChannelId, lTransferToSessionId,  nTransferToChannelId, sTransferer, sTransferee, sTransferTo);
		return;
	}

	m_pVaxTransfer->OnCallSessionTransferConsult(lTransfererSessionId, nTransfererChannelId, lTransferToSessionId,  nTransferToChannelId, sTransferer, sTransferee, sTransferTo);
}

void CVaxCallSession::OnCallSessionTransfering(LONG lSessionId, LONG lTransferType, int nChannelId, LONG lStatusCode, LPCTSTR sReasonPhrase)
{
	m_pVaxTransfer->OnCallSessionTransfering(lSessionId, lTransferType, nChannelId, lStatusCode, sReasonPhrase);
}

void CVaxCallSession::OnCallSessionTransferFailed(LONG lSessionId, LONG lTransferType, int nChannelId, LONG lStatusCode, LPCTSTR sReasonPhrase)
{
	m_pVaxTransfer->OnCallSessionTransferFailed(lSessionId, lTransferType, nChannelId, lStatusCode, sReasonPhrase);
} 

void CVaxCallSession::OnCallSessionTransferTimeout(LONG lSessionId, LONG lTransferType, int nChannelId)
{
	m_pVaxTransfer->OnCallSessionTransferTimeout(lSessionId, lTransferType, nChannelId);
} 

void CVaxCallSession::OnCallSessionTransfered(LONG lSessionId, LONG lTransferType)
{
	m_pVaxTransfer->OnCallSessionTransfered(lSessionId, lTransferType);
}

//////////////////////////////////////////////////////////////////////////
//////////////////////////  CVAXDTMF//////////////////////////////////

void CVaxCallSession::OnDetectedDigitDTMF(LONG lSessionId, int nChannelId, LPCTSTR sDigitDTMF)
{
	m_pVaxDTMF->OnDetectedDigitDTMF(lSessionId, nChannelId, sDigitDTMF);
}

//////////////////////////////////////////////////////////////////////////
//////////////////////////  CVAXDialTone  ////////////////////////////////

void CVaxCallSession::OnCallSessionDialToneStart(LONG lSessionId, int nChannelId, LONG lWaveId)
{
	m_pVaxDialTone->OnCallSessionDialToneStart(lSessionId, nChannelId, lWaveId);
}

void CVaxCallSession::OnCallSessionDialToneStop(LONG lSessionId, int nChannelId, LONG lWaveId)
{
	m_pVaxDialTone->OnCallSessionDialToneStop(lSessionId, nChannelId, lWaveId);
}

void CVaxCallSession::OnCallSessionPlayWaveDone(LONG lSessionId, int nChannelId, LONG lWaveId)
{
}

//////////////////////////////////////////////////////////////////////////
//////////////////////////  CVAXDETECTEDAMD  /////////////////////////////

void CVaxCallSession::OnCallSessionDetectedAMD(LONG lSessionId, int nChannelId, BOOL bIsHuman)
{
	m_pVaxDetectedAMD->OnCallSessionDetectedAMD(lSessionId, nChannelId, bIsHuman);
}

/////////////////////////////////////////////////////////////////////////
//////////////////////////  CVAXRecordedPCM  ////////////////////////////

void CVaxCallSession::OnCallSessionRecordedPCM(LONG lSessionId, int nChannelId, char* pDataPCM, LONG lSizePCM)
{
	m_pVaxRecordPCM->OnCallSessionRecordedPCM(lSessionId, nChannelId, pDataPCM, lSizePCM);
}

/////////////////////////////////////////////////////////////////////////
//////////////////////////  CVAXQueue ////////////////////////////

void CVaxCallSession::OnAddCallSessionToQueueSuccess(LPCTSTR sQueueName, LONG lSessionId)
{
	m_pVaxQueue->OnAddCallSessionToQueueSuccess(sQueueName, lSessionId);
}

void CVaxCallSession::OnAddCallSessionToQueueFailed(LPCTSTR sQueueName, LONG lSessionId)
{
	m_pVaxQueue->OnAddCallSessionToQueueFailed(sQueueName, lSessionId);
}

void CVaxCallSession::OnQueuePlayWaveStart(int nEventType, LPCTSTR sQueueName, LONG lSessionId)
{
	m_pVaxQueue->OnQueuePlayWaveStart(nEventType, sQueueName, lSessionId);
}

void CVaxCallSession::OnQueuePlayWaveStop(int nEventType, LPCTSTR sQueueName, LONG lSessionId)
{
	m_pVaxQueue->OnQueuePlayWaveStop(nEventType, sQueueName, lSessionId);
}

/////////////////////////////////////////////////////////////////////////
//////////////////////////  CVaxRingGroup  ////////////////////////////

void CVaxCallSession::OnAddCallSessionToRingGroupSuccess(LONG lSessionId, LPCTSTR sRingGroupName)
{
	m_pVaxRingGroup->OnAddCallSessionToRingGroupSuccess(lSessionId, sRingGroupName);
}

void CVaxCallSession::OnAddCallSessionToRingGroupFailed(LONG lSessionId, LPCTSTR sRingGroupName)
{
	m_pVaxRingGroup->OnAddCallSessionToRingGroupFailed(lSessionId, sRingGroupName);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

BOOL CVaxCallSession::AcceptCallSession(LONG lSessionId, LPCTSTR sCallerName, LPCTSTR sCallerId, LPCTSTR sDialNo, int nToPeerType, LPCTSTR sToPeerName)
{
	BOOL bResult = CVaxServerExCOM::m_pVaxServerExCOM->AcceptCallSession(lSessionId, sCallerName, sCallerId, sDialNo, nToPeerType, sToPeerName, 30);
	
	if(bResult == FALSE) 
	{	
		CVaxServerExCOM::m_pVaxServerExCOM->RejectCallSession(lSessionId, 404, "Not Found");
		return bResult;
	}
		
	return bResult;
}

