using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

using ServerSIP;

namespace VaxTeleConfigSIP
{
    public partial class MainForm : Form
    {
        public static MainForm m_objMainForm;

        cVaxServerSIP m_objVaxTeleSIP = new cVaxServerSIP();

        private GeneralForm m_objGeneralForm;
        private LoginForm m_objLoginForm;
        private LineForm m_objLineForm;
        private RoomForm m_objRoomForm;
        private DirectProxyForm m_objDirectProxyForm;
        private QueueForm m_objQueueForm;
        private RingGroupForm m_objRingGroupForm;
        private CallPickupForm m_objCallPickupForm;
       
        private ErrorLogForm m_objErrorLogForm;
        private StatusLogForm m_objStatusLogForm;

        public MainForm()
        {
            m_objGeneralForm = new GeneralForm(ref m_objVaxTeleSIP);

            m_objLoginForm = new LoginForm(ref m_objVaxTeleSIP);
            m_objLineForm = new LineForm(ref m_objVaxTeleSIP);

            m_objDirectProxyForm = new DirectProxyForm(ref m_objVaxTeleSIP);
            m_objRoomForm = new RoomForm(ref m_objVaxTeleSIP);

            m_objQueueForm = new QueueForm(ref m_objVaxTeleSIP);
            m_objRingGroupForm = new RingGroupForm(ref m_objVaxTeleSIP);
            m_objCallPickupForm = new CallPickupForm(ref m_objVaxTeleSIP);
                                  
            m_objErrorLogForm = new ErrorLogForm();
            m_objStatusLogForm = new StatusLogForm();

            m_objVaxTeleSIP.m_objMainFormDlg = this;

            m_objMainForm = this;
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            m_objGeneralForm.TopLevel = false;
            m_objGeneralForm.Visible = true;
            m_objGeneralForm.FormBorderStyle = FormBorderStyle.None;
            m_objGeneralForm.Dock = DockStyle.Fill;
            TabCtrlMain.TabPages[0].Controls.Add(m_objGeneralForm);

            m_objLoginForm.TopLevel = false;
            m_objLoginForm.Visible = true;
            m_objLoginForm.FormBorderStyle = FormBorderStyle.None;
            m_objLoginForm.Dock = DockStyle.Fill;
            TabCtrlMain.TabPages[1].Controls.Add(m_objLoginForm);

            m_objLineForm.TopLevel = false;
            m_objLineForm.Visible = true;
            m_objLineForm.FormBorderStyle = FormBorderStyle.None;
            m_objLineForm.Dock = DockStyle.Fill;
            TabCtrlMain.TabPages[2].Controls.Add(m_objLineForm);

            m_objDirectProxyForm.TopLevel = false;
            m_objDirectProxyForm.Visible = true;
            m_objDirectProxyForm.FormBorderStyle = FormBorderStyle.None;
            m_objDirectProxyForm.Dock = DockStyle.Fill;
            TabCtrlMain.TabPages[3].Controls.Add(m_objDirectProxyForm);

            m_objRoomForm.TopLevel = false;
            m_objRoomForm.Visible = true;
            m_objRoomForm.FormBorderStyle = FormBorderStyle.None;
            m_objRoomForm.Dock = DockStyle.Fill;
            TabCtrlMain.TabPages[4].Controls.Add(m_objRoomForm);

            m_objQueueForm.TopLevel = false;
            m_objQueueForm.Visible = true;
            m_objQueueForm.FormBorderStyle = FormBorderStyle.None;
            m_objQueueForm.Dock = DockStyle.Fill;
            TabCtrlMain.TabPages[5].Controls.Add(m_objQueueForm);


            m_objRingGroupForm.TopLevel = false;
            m_objRingGroupForm.Visible = true;
            m_objRingGroupForm.FormBorderStyle = FormBorderStyle.None;
            m_objRingGroupForm.Dock = DockStyle.Fill;
            TabCtrlMain.TabPages[6].Controls.Add(m_objRingGroupForm);

            m_objCallPickupForm.TopLevel = false;
            m_objCallPickupForm.Visible = true;
            m_objCallPickupForm.FormBorderStyle = FormBorderStyle.None;
            m_objCallPickupForm.Dock = DockStyle.Fill;
            TabCtrlMain.TabPages[7].Controls.Add(m_objCallPickupForm);

            m_objErrorLogForm.TopLevel = false;
            m_objErrorLogForm.Visible = true;
            m_objErrorLogForm.FormBorderStyle = FormBorderStyle.None;
            m_objErrorLogForm.Dock = DockStyle.Fill;
            TabCtrlMain.TabPages[8].Controls.Add(m_objErrorLogForm);

            m_objStatusLogForm.TopLevel = false;
            m_objStatusLogForm.Visible = true;
            m_objStatusLogForm.FormBorderStyle = FormBorderStyle.None;
            m_objStatusLogForm.Dock = DockStyle.Fill;
            TabCtrlMain.TabPages[9].Controls.Add(m_objStatusLogForm);

        }

        private void TabCtrlMain_DrawItem(object sender, DrawItemEventArgs e)
        {
            Graphics g = e.Graphics;
            Font f = new Font(e.Font, FontStyle.Bold);
            SolidBrush b = new SolidBrush(SystemColors.Control);
            g.FillRectangle(b, e.Bounds);

            b.Color = Color.Black;
            g.DrawString(this.TabCtrlMain.TabPages[e.Index].Text, e.Font, b, e.Bounds.X + 2, e.Bounds.Y + 2);

            b.Dispose();
            f.Dispose();
        }

        private void TabCtrlMain_SelectedIndexChanged(object sender, EventArgs e)
        {
            int nTabIndex = TabCtrlMain.SelectedIndex;

            if(nTabIndex == 0)
                m_objGeneralForm.OnSetActive();
            else if(nTabIndex == 1)
                m_objLoginForm.OnSetActive();
            else if(nTabIndex == 2)
                m_objLineForm.OnSetActive();
            else if(nTabIndex == 3)
                m_objDirectProxyForm.OnSetActive();//
            else if(nTabIndex == 4)
                m_objRoomForm.OnSetActive();//
            else if(nTabIndex == 5)
                m_objQueueForm.OnSetActive();
            else if(nTabIndex == 6)
                m_objRingGroupForm.OnSetActive();
            else if(nTabIndex == 7)
                m_objCallPickupForm.OnSetActive();
        }

        private void MainForm_Closing(object sender, FormClosingEventArgs e)
        {
            m_objVaxTeleSIP.UnInitialize();
        }

        public void SetPageTitle(string sTitle, int nPageNo)
        {
            TabCtrlMain.TabPages[nPageNo].Text = sTitle;
        }

        /////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////

        delegate void StatusLog(string sMsg);
        delegate void ErrorLog(string sMsg);

        public void PostStatusLog(string sMsg)
        {
            StatusLog objEventStatusLog = new StatusLog(OnStatusLog);
            this.BeginInvoke(objEventStatusLog, sMsg);

        }
        public void PostErrorLog(string sMsg)
        {
            ErrorLog objEventErrorLog = new ErrorLog(OnErrorLog);
            this.BeginInvoke(objEventErrorLog, sMsg);
      
        }

        public void OnStatusLog(string sStatusMsg)
        {
            m_objStatusLogForm.OnStatusLog(sStatusMsg); 
        }
        
        public void OnErrorLog(string sErrorMsg)
        {
            m_objErrorLogForm.OnErrorLog(sErrorMsg);
        }
    }
}