﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Win32;

namespace ServerSIP
{
    class cStoreRegistry
    {
        const string REGISTRY_ROOT_PATH = "SOFTWARE\\VaxTeleServerSIP";

        private RegistryKey m_hKey;

        public cStoreRegistry()
        {
            Open();
        }

        ~cStoreRegistry()
        {
            Close();
        }

        ////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////

        public void Open()
        {
            if (m_hKey != null)
                return;

            m_hKey = Registry.CurrentUser.CreateSubKey(REGISTRY_ROOT_PATH);
        }

        public void Close()
        {
            if(m_hKey != null)
               m_hKey.Close();
        }

        ////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////

        private RegistryKey CreateKey(string sKeyName)
        {
            if (m_hKey == null) 
                return null;
            
            if (sKeyName.Length == 0)
                return null;

            return m_hKey.CreateSubKey(sKeyName);
        }

        private RegistryKey OpenKey(string sKeyName, Boolean bWritable)
        {
            if (m_hKey == null)
                return null;
            
            if (sKeyName.Length == 0)
                return null;

            return m_hKey.OpenSubKey(sKeyName, bWritable);
        }
        
        //////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////

        public Boolean SetTEXT(string sKeyName, string sValueName, string sValueData)
        {
            if (sValueName.Length == 0) return false;
            
            RegistryKey hKey = CreateKey(sKeyName);
            if (hKey == null)   return false;
            
            hKey.SetValue(sValueName, sValueData, RegistryValueKind.String);
            hKey.Close();

            return true;
        }

        public Boolean SetINT(string sKeyName, string sValueName, int nValueData)
        {
            if (sValueName.Length == 0) return false;
            
            RegistryKey hKey = CreateKey(sKeyName);
            if (hKey == null) return false;
            
            hKey.SetValue(sValueName, nValueData, RegistryValueKind.DWord);
            hKey.Close();

            return true;
        }

        public Boolean SetLONG(string sKeyName, string sValueName, long lValueData)
        {
            RegistryKey hKey = CreateKey(sKeyName);
            
            if (hKey == null) return false;

            if (sValueName.Length == 0) return false;

            hKey.SetValue(sValueName, lValueData, RegistryValueKind.DWord);

            hKey.Close();
            return true;
        }

        public Boolean SetBOOL(string sKeyName, string sValueName, bool bValueData)
        {
            int nValueData = 0;

            if (bValueData)
                nValueData = 1;

            return this.SetINT(sKeyName, sValueName, nValueData);
        }

        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////
       
        public Boolean GetTEXT(string sKeyName, string sValueName, ref string sValueData)
        {
            if (sValueName.Length == 0) return false;

            RegistryKey hKey = OpenKey(sKeyName, false);
            if (hKey == null) return false;

            string sResultValueData = (string)hKey.GetValue(sValueName, null);

            if (sResultValueData == null)
            {
                hKey.Close();
                return false;
            }

            sValueData = (string) sResultValueData;
            sValueData = sValueData.Trim();

            hKey.Close();
            return true;
        }

        public Boolean GetINT(string sKeyName, string sValueName, ref int nValueData)
        {
            if (sValueName.Length == 0) return false;
            
            RegistryKey hKey = OpenKey(sKeyName, false);
            if (hKey == null) return false;
            
            nValueData = -1;
            nValueData = (int)hKey.GetValue(sValueName, -1);
            
            hKey.Close();
            return true;
        }

        public Boolean GetLONG(string sKeyName, string sValueName, ref long lValueData)
        {
            if (sValueName.Length == 0) return false;

            RegistryKey hKey = OpenKey(sKeyName, false);         
            if (hKey == null) return false;

            lValueData = (long)hKey.GetValue(sValueName, -1);
            hKey.Close();

            return true;
        }

        public Boolean GetBOOL(string sKeyName, string sValueName, ref bool bValueData)
        {
            int nValueData = 0;
                        
            if (!GetINT(sKeyName, sValueName, ref nValueData))
                return false;

            if (nValueData == 0)
                bValueData = false;
            else
                bValueData = true;

            return true;
        }

        public bool DeleteSubKey(string sSubKeyName)
        {
            if (sSubKeyName.Length == 0)
                return false;

            m_hKey.DeleteSubKeyTree(sSubKeyName, false);
            return true;
        }

        public bool DeleteValue(string sKeyRootName, string sValueName)
        {
            if (sValueName.Length == 0)
                return false;

            RegistryKey hKey = OpenKey(sKeyRootName, true);
            if (hKey == null) return false;

            hKey.DeleteValue(sValueName, false);
            hKey.Close();

            return true;
        }

        public Boolean EnumSubKeys(string sKeyRootName, ref List<string> objListSubKeys)
        {
            RegistryKey hKey = OpenKey(sKeyRootName, false);
            if (hKey == null) return false;

            string[] arrayLogin = (string[])hKey.GetSubKeyNames();

            for (int nCount = 0; nCount < arrayLogin.Length; nCount++)
                objListSubKeys.Add(arrayLogin[nCount]);

            hKey.Close();

            return true;
        }

        public bool IsKeyExist(string sSubKeyName)
        {
            RegistryKey hKey = OpenKey(sSubKeyName, false);
            if (hKey == null) return false;

            hKey.Close();
            return true;
        }
    }
}
